# SPDX-FileCopyrightText: 2021-2024 SanderTheDragon <sanderthedragon@zoho.com>
#
# SPDX-License-Identifier: MIT

"""
This file just contains the version for Hatch.
It will also be used as extension version.
"""

__version__ = '1.1.0'
